/*
  PromptNuFlux example_flux.cc
  Author: Luca Rottoli, 2015
  R. Gauld, J. Rojo, L. Rottoli, J. Talbert,             arXiv:1506.08025
  R. Gauld, J. Rojo, L. Rottoli, S. Sarkar, J. Talbert,  arXiv:1511.06346
*/

#include <iostream>
#include <cstring>
#include <string>
#include <fstream>
#include <iomanip>
#include <sstream>
#include <sys/time.h>
#include <sys/stat.h>
#include <getopt.h> 
#include <cstdlib>

#include "src/flux_interpolation.hh"

using namespace std;

void print_usage() {
  cout << endl
       << "Usage:" << endl
       << " -e  --e <e>  set min Energy e. Energy allowed: 10^3 < e < 10^7.5" << endl
       << " -E  --E <E>  set max Energy E. Energy allowed: 10^3 < E < 10^7.5" << endl
       << " -f  --f <f>  set Cosmic Ray Flux model. Insert an integer from 0 to 4. 0: BPL; 1: H3P, 2: H3A, 3: H14A, 4: H14B" << endl
       << endl;
  exit(0);
  return;
}

void read_arguments(int argc, char* argv[], double &Emin, double &Emax, fluxmodel &flux) {
  const char* const short_options = "he:E:f:";
  const struct option long_options[] = { { "help", 0, NULL, 'h' },
					 { "e", 1, NULL, 'e' },
           { "E", 1, NULL, 'E' },
           { "f", 1, NULL, 'f' },
					 { NULL, 0, NULL, 0 } };
  int next_option;
  do {
    next_option = getopt_long (argc, argv, short_options, long_options, NULL);
    switch (next_option) {
    case 'h':
      print_usage();
    case 'e':
      Emin = strtod(optarg, NULL);
      break;
    case 'E':
      Emax = strtod(optarg, NULL);
      break;
    case 'f':
      flux = (fluxmodel) strtol(optarg, NULL, 10);
      break;
    case '?':
      print_usage();
    case -1: break;
    default: abort();
    }
  }
  while (next_option != -1);
  return;
}


/*
 * Main program
 */
int main(int argc, char* argv[])
{

  if(argc==1) cout << endl << "\033[1;32m Hint: Run with -h to access options\033[0m" << endl << endl;
  
  struct timeval t0, t1;
  gettimeofday(&t0,NULL);

  string mod[] = {"BPL","H3P", "H3A", "H14A", "H14B"};

  // default values
  double Emin = 1e3;
  double Emax = pow(10,7.5);
  fluxmodel flux = BPL;

  // read runtime arguments
  read_arguments(argc, argv, Emin, Emax, flux);
  if (flux > 4)
  {
  	cout << endl << "\033[1;31m Insert a correct value for the cosmic ray flux model. Abort. \033[0m" << endl << endl;      
    abort();
  }

  // print parameters
  cout.precision(2);
  cout << scientific;
  if(Emin >= Emax)
  {
    cout << endl << "\033[1;31m Emin " << Emin << " must be less than Emax " << Emax << " Abort. \033[0m" << endl << endl;  
    abort();
  }
  cout << endl << "Emin: " << "1e" << log10(Emin) << endl;
  cout <<         "Emax: " << "1e" << log10(Emax) << endl;
  cout << endl << "Cosmic Ray Flux Model: "       << mod[flux] << endl << endl;

  // Initialize FluxInterpolation
  FluxInterpolation * PromptFlux = new FluxInterpolation;

  //Print Results
  //If you modify the name of the output and its layout
  //you will need to modify accordingly also plotter.py
  //to be able to plot
  string output = "outputs/" + mod[flux] +"flux.dat";
  cout << endl << "Writing results in " << output << endl << endl;
  ofstream out(output.c_str());
  out << "CRF model " << mod[flux] << endl;
  out << "prompt atmospheric neutrino flux [E^3 Phi(GeV^2/(cm^2 s sr))]" << endl;
  out << setw(10) << "E" << setw(15) << "CentralValue" << setw(15) << "UpperValue" << setw(15) << "LowerValue" << endl;
  for (int i = 0; i <= 100; ++i)
  {
  	double logE = log10(Emin) + (log10(Emax)-log10(Emin))/100.*i;
  	double E = pow(10,logE);
    out.precision(3);
    out << scientific;
    out << setw(10) << E 
        << setw(15) << PromptFlux->CentralFlux(E,flux) 
        << setw(15) << PromptFlux->UpperFlux(E,flux)
        << setw(15) << PromptFlux->LowerFlux(E,flux)
        << endl;
  }
  out.close();


  gettimeofday(&t1,NULL);
  double t=t1.tv_sec-t0.tv_sec+(t1.tv_usec-t0.tv_usec)*0.000001;
  double cput = clock() / (float)CLOCKS_PER_SEC;
  cout << "Real time elapsed:  " << t    << " s" << endl;
  cout << "CPU  time elapsed:  " << cput << " s" << endl;
  
  return 0;
  
}